/*
 * @(#) DSOLApplication.java Oct 24, 2003 Copyright (c) 2002-2005 Delft
 * University of Technology Jaffalaan 5, 2628 BX Delft, the Netherlands. All
 * rights reserved. This software is proprietary information of Delft University
 * of Technology The code is published under the Lesser General Public License
 */
package nl.tudelft.simulation.dsol.gui;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.ConsoleHandler;

import nl.tudelft.simulation.dsol.experiment.Experiment;
import nl.tudelft.simulation.dsol.experiment.ExperimentalFrame;
import nl.tudelft.simulation.dsol.gui.applets.DSOLSecurityManager;
import nl.tudelft.simulation.dsol.gui.windows.DSOLFrame;
import nl.tudelft.simulation.dsol.simulators.Animator;
import nl.tudelft.simulation.dsol.simulators.SimulatorInterface;
import nl.tudelft.simulation.event.EventProducer;
import nl.tudelft.simulation.language.io.URLResource;
import nl.tudelft.simulation.logger.Logger;
import nl.tudelft.simulation.logger.handlers.MemoryHandler;

/**
 * The DSOL Application <br>
 * (c) copyright 2002-2005 <a href="http://www.simulation.tudelft.nl">Delft
 * University of Technology </a>, the Netherlands. <br>
 * See for project information <a
 * href="http://www.simulation.tudelft.nl">www.simulation.tudelft.nl </a> <br>
 * License of use: <a href="http://www.gnu.org/copyleft/lesser.html">Lesser
 * General Public License (LGPL) </a>, no warranty.
 * 
 * @version $Revision: 1.13 $ $Date: 2005/08/09 10:41:04 $
 * @author <a href="http://www.peter-jacobs.com">Peter Jacobs </a>
 */
public class DSOLApplication extends EventProducer implements
        DSOLApplicationInterface
{
    /** the experiment */
    protected ExperimentalFrame experimentalFrame = null;

    /** the properties */
    protected Properties properties = new Properties();

    /** the recents */
    protected Set<Recent> recents = new TreeSet<Recent>(new RecentComparator());

    /** the simulator to use */
    private SimulatorInterface simulator = null;

    /**
     * constructs a new DSOLApplicationInterface
     * 
     * @param navigation an URL to an xml-based navigation structure.
     */
    public DSOLApplication(final URL navigation)
    {
        super();
        System.setSecurityManager(new DSOLSecurityManager());
        // Runtime.runFinalizersOnExit(true);
        Logger.setDefaultHandler(MemoryHandler.class);
        try
        {
            this.readProperties(this.properties);
            new DSOLFrame(this, navigation);
        } catch (Exception exception)
        {
            Logger.warning(this, "DSOLApplicationInterface", exception);
        }
    }

    /**
     * @see java.lang.Object#finalize()
     */
    @Override
    protected void finalize() throws Throwable
    {
        this.writeProperties(this.properties);
        super.finalize();
    }

    /**
     * returns the experiment
     * 
     * @return ExperimentalFrame
     */
    public ExperimentalFrame getExperimentalFrame()
    {
        return this.experimentalFrame;
    }

    public SimulatorInterface getSimulator()
    {
        return this.simulator;
    }

    /**
     * sets the experiment
     * 
     * @param experimentalFrame the experiment
     */
    public void setExperimentalFrame(final ExperimentalFrame experimentalFrame)
    {
        this.experimentalFrame = experimentalFrame;
        if (this.experimentalFrame != null
                && this.experimentalFrame.getUrl() != null)
        {
            this.recents.add(new Recent(experimentalFrame.getUrl().toString(),
                    System.currentTimeMillis()));
            Recent[] elements = this.recents.toArray(new Recent[this.recents
                    .size()]);
            this.recents.clear();
            this.recents.addAll(Arrays.asList(elements));
            try
            {
                this.writeProperties(this.properties);

                this.simulator = new Animator();
                for (Experiment experiment : this.experimentalFrame
                        .getExperiments())
                {
                    experiment.setSimulator(this.simulator);
                }
            } catch (Exception exception)
            {
                Logger.warning(this, "setExperiment", exception);
            }
        }
        super.fireEvent(EXPERIMENT_CHANGED_EVENT, experimentalFrame);
    }

    /**
     * returns the properties
     * 
     * @return Properties
     */
    public Properties getProperties()
    {
        return this.properties;
    }

    /**
     * executes the dsol control panel
     * 
     * @param args the command-line arguments
     */
    public static void main(final String[] args)
    {
        Logger.setDefaultHandler(ConsoleHandler.class);
        URL navigation = null;
        if (args.length != 0)
        {
            navigation = URLResource.getResource(args[0]);
        } else if (System.getProperty("dsol.navigation") != null)
        {
            navigation = URLResource.getResource(System
                    .getProperty("dsol.navigation"));
        }
        if (navigation == null)
        {
            navigation = URLResource.getResource("/navigation.xml");
        }
        new DSOLApplication(navigation);
    }

    /**
     * reads the properties
     * 
     * @param properties the properties to read
     * @throws IOException on failure
     */
    private void readProperties(final Properties properties) throws IOException
    {
        properties.load(URLResource.getResourceAsStream("/dsol.properties"));
        String tempPath = System.getProperty("java.io.tmpdir");
        String fileName = this.getClass().getName() + "_dsol.properties";
        File file = new File(tempPath, fileName);
        if (file.exists() && !("true".equals(properties.get("edited"))))
        {
            FileInputStream stream = new FileInputStream(file);
            properties.load(stream);
            stream.close();
        }
        int i = 0;
        String value = null;
        while ((value = properties.getProperty("recent[" + i + "]")) != null)
        {
            String[] elements = value.split("!#!");
            this.recents.add(new Recent(elements[0], Long
                    .parseLong(elements[1])));
            i++;
        }
    }

    /**
     * reads the properties
     * 
     * @param properties the properties to read
     * @throws IOException on failure
     */
    private void writeProperties(final Properties properties)
            throws IOException
    {
        int loop = 0;
        for (Recent recent : this.recents)
        {
            if (loop <= 5)
            {
                this.properties.put("recent[" + loop + "]", recent.getLabel()
                        + "!#!" + recent.getExecutionTime());
            }
            loop++;
        }
        String tempPath = System.getProperty("java.io.tmpdir");
        String fileName = this.getClass().getName() + "_dsol.properties";
        File file = new File(tempPath, fileName);
        if (!file.exists())
        {
            file.createNewFile();
        }
        FileOutputStream outputStream = new FileOutputStream(file);
        this.properties.store(outputStream, "");
        outputStream.close();
    }

    /**
     * the Recent Comparator
     */
    private static class RecentComparator implements Comparator<Recent>
    {
        /**
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        public int compare(final Recent r1, final Recent r2)
        {
            if (r1.getLabel().equals(r2.getLabel()))
            {
                long executionTime = Math.max(r1.getExecutionTime(), r2
                        .getExecutionTime());
                r1.executionTime = executionTime;
                r2.executionTime = executionTime;
                return 0;
            }
            return new Long(r2.getExecutionTime()).compareTo(new Long(r1
                    .getExecutionTime()));
        }
    }

    /**
     * A RecentItem
     */
    private static class Recent
    {
        /** the label to use */
        private String label = null;

        /** the executionTime */
        protected long executionTime = 0L;

        /**
         * constructs a new Recent
         * 
         * @param label the label of the experimen
         * @param executionTime the executionTime of the experiment
         */
        public Recent(final String label, final long executionTime)
        {
            this.executionTime = executionTime;
            this.label = label;
        }

        /**
         * @return Returns the executionTime.
         */
        public long getExecutionTime()
        {
            return this.executionTime;
        }

        /**
         * @return Returns the label.
         */
        public String getLabel()
        {
            return this.label;
        }
    }
}